//	Temperaturschalter, Harald Sattler, 11/2019
// 
// Der ESP01 misst ueber einen DS18b20 OneWire-Temperatursensor die Temperatur des 
// ueberwachten Photovoltaik-Wechselrichters und schaltet bei Ueberschreitung des im
// EEPROM hinterlegten Grenzwertes (maxTempAllowed) ueber ein SolidState-Relais einen Luefter ein.
// Unterschreitet die Temperatur den Einschaltwert abzueglich einer  
// definierten Hysterese (TempHysterese), wird der Luefter wieder abgeschaltet.
// Die gemessene Temperatur sowie der Status des Schaltausgangs wir auf einem ueber 
// I2C angeschlossenen OLED-Display angezeigt.
//
// 2019-05-23, V1.0
// - initiale Version, Funktionalitaet wie oben beschrieben, Parameter fest im Programm hinterlegt
//
// 2019-11-21, V2.0
// - Erweiterung des WEB-Interface um Einstellmoeglichkeiten fuer Einschalttemperatur und Hysterese
//   Die Anzeige der Einstellung der Parameter kann ein/ausgeschaltet werden
// - Speicherung der Parameter "Einschalttemperatur" und "Hysterese" im Flash (EEPROM-Ersatz)
// - Messung triggern und Sensor auslesen zeitlich getrennt um die Reaktionszeit des WEB Interface zu minimieren
// - Code aufgeraeumt, Kommentare erweitert
// - Bei Bedienung per Handy (Firefox)  haengt sich der Server auf :(
//
// 2019-11-25
// - Code fuer Wemos D1 erweitert um Debugausgaben uebertragen und ohne Umstecken entwickeln zu koennen
// - wenn "warten auf Client" (debugOUT("8a")) zu lange dauert abbrechen und neu aufsetzen
//
// 2019-11-26
// - Das OLED-Display zeigt bereits nach wenigen Monaten Abnutzungsspuren, Pixel, die immer an waren, sind 
//   deutlich dunkler als nicht oder wenig benutzte Pixel 
//   (-> Taster zum Display aktivieren, nach x Sekunden wieder ausschalten (geht nicht, keine Pins frei))
//   -> Display nach WEB-Interface-Benutzung aktivieren,  nach x Sekunden wieder ausschalten
//   Die Ausschaltzeit kann ueber Einstellungen geaendert werden und wird im EEPROM gespeichert  

#include <WiFiUdp.h>
#include <WiFiServerSecureBearSSL.h>
#include <WiFiServerSecureAxTLS.h>
#include <WiFiServerSecure.h>
#include <WiFiServer.h>
#include <WiFiClientSecureBearSSL.h>
#include <WiFiClientSecureAxTLS.h>
#include <WiFiClientSecure.h>
#include <WiFiClient.h>
#include <ESP8266WiFiType.h>
#include <ESP8266WiFiSTA.h>
#include <ESP8266WiFiScan.h>
#include <ESP8266WiFiMulti.h>
#include <ESP8266WiFiGeneric.h>
#include <ESP8266WiFiAP.h>
#include <CertStoreBearSSL.h>
#include <BearSSLHelpers.h>
#include <ESP8266WiFi.h>

#include <DallasTemperature.h>								// Anbindung des Temperatursensors
#include <Wire.h>											// OLED IIC Mode 
#include <EEPROM.h>											// Speicherung der einstellbaren Parameter im Flash

//#define DEBUG												// Abschalten des I2C Interface fuer das OLED, damit serielle Debugausgaben erfolgen koennen
															// Beide Interfaces liegen beim ESP-01 auf demselben Pin (der hat nur 2 frei ;-)
#define ESP-01												// Code fuer ESP-01 compilieren (andere Pin-Benennung als beim Wemos D1)


// wo ist der Code gerade unterwegs?
char i = 0;
#define NOCLIENT 1											// kein Client angemeldet
#define TIMEOUT 2											// Timeout der Clientverbindung
#define GETTEMP 3											// Aufruf GetTemp()
#define INLOOP 4											// in der loop()
#define WIFISTART 5											// Wifi starten
#define WNOTCLIENT 6										// while no client

const char* ssid = "SSID";									// eigene WLAN ssid eintragen
const char* password = "Password";							// eigenes WLAN password eintragen

#ifdef ESP-01
// Festlegung der IOs
const char relaisOut = 2;									// GPIO0 des ESP8266
const char ledPin = 2;

#ifndef DEBUG
// Definitionen fuer I2C & OLED
#define PIN_I2C_SDA 1										// TX-Pin
#define PIN_I2C_SCL 3										// RX-Pin
#endif
// data wire des OneWire Bus GPIO2:
#define ONE_WIRE_BUS 0

#else
const char relaisOut = 4;									// GPIO0 des ESP8266
const char ledPin = 4;

#ifndef DEBUG
// Definitionen fuer I2C & OLED
#define PIN_I2C_SDA 5										// TX-Pin
#define PIN_I2C_SCL 0										// RX-Pin
#endif

// data wire des OneWire Bus GPIO2:
#define ONE_WIRE_BUS 14
#endif

// Definitionen fuer I2C & OLED
#define OLED_address  0x3c									// I2C-Adresse des OLED Display

// globale Variablen
char tempBuf[20];											// temporaerer Puffer fuer OLED Ausgaben
char TemperaturBuffer[6];									//  Puffer fuer String-Ausgabe, enthaelt die Temperatur
String Temperatur;
String IPadress;	
char OLEDBuffer[20];										// fuer Ausgabe auf OLED
bool value = false;											// Merker fuer Luefter an/aus
float currentTemp = 0;										// die aktuell vom Sensor erfasste Temperatur
bool wlanOverride = false;									// wurde ueber WLAN der Luefter ein-/ausgeschaltet? Dann Temperatur ignorieren
const char ZeilenOffset = 2;								// linker Rand fuer Textausgaben auf OLED

float maxTempAllowed = 25.0;								// Einschalttemperatur fuer den Luefter
float TempHysterese = 0.5;									// wieder ausschalten, wenn darunter

unsigned long sensorTriggerZeit = 0;						// Zeitpunkt Sensor-Request (erfasse Temperatur)
bool sensorTriggered = false;								// wurde eine Messung ausgeloest?
DeviceAddress sensorAddress;								// Speicher fuer Device Adresse des Temp Sensors

bool bEinstellungen = false;								// nach Click auf "Einstellungen" anderer Seiteninhalt
int emptyCheck = 0;											// check if EEPROM has been set already, emptyCheck sitzt auf EEPROM Adr.0
char cEEPROMchanged = 0;									// Flag, true wenn EEPROM Inhalte geaendert wurden und gesichert werden muessen
const char TEMP = 1;										// Flag fuer geaenderte Einschalttemperatur
const char HYST = 2;										// Flag fuer geaenderte Hysterese
const char DISP = 3;
const char NOCHANGE = 0;									// keine Aenderung

const int EEPROMFIRSTENTRY = 10;							// erster Eintrag im EEPROM

unsigned long displayTimer = 0;								// Timer zum Ausschalten des OLED Display
int iDisplayTime = 10000;									// nach 10 s Display dunkel schalten
bool bDisplayIsOn = false;

// Instantiierungen
WiFiServer server(80);										// WEB server instantiieren

OneWire oneWire(ONE_WIRE_BUS);								// OneWire fuer DS18b20 Temperatur Sensor instantiieren
DallasTemperature sensors(&oneWire);						// Sensor instantiieren

// Unterprogramme ------------------------------------------------------------------------------------------------------------------
void triggerTemperature()									// Routine startet die Temperaturmessung
{
	if (sensorTriggered == false)
	{
		sensors.requestTemperaturesByIndex(0);				// request temperature
		sensorTriggerZeit = millis();						// 
		sensorTriggered = true;
	}
}
void getTemperature()										// Sensor: Messung triggern und wenn fertig Temperatur auslesen
{
	if ((sensorTriggered == true) && (millis() > sensorTriggerZeit + 800))	// bei hoechster Aufloesung benoetigt der DS18B ~750 ms bis der Wert gelesen werden kann
	{
		//currentTemp = sensors.getTempCByIndex(0);			// Temperatur in globaler Var ablegen
		currentTemp = sensors.getTempC(sensorAddress);		// Temperatur in globaler Var ablegen
		sensorTriggered = false;
	}
}

String getContent()											// Temperatur als String fuer Ausgabe auf WEB Interface
{
	dtostrf(currentTemp, 2, 1, TemperaturBuffer);				// convert to string
	dtostrf(currentTemp, 2, 1, OLEDBuffer);					// Temperatur als Float in den OLEDBuffer kopieren. 4-stellig: "27.8"

	String content = TemperaturBuffer;						// create content for the website
	return content;
}

void calcTempAndSetRelaisVar()
{
	if (wlanOverride == false) {
		if (currentTemp > maxTempAllowed)					{value = true;}		// Relais einschalten
		if (currentTemp < (maxTempAllowed - TempHysterese))	{value = false;}	// Relais ausschalten
	}
}

void WiFiStart()
{
	// Connect to WiFi network
	Serial.print("Connecting to ");
	Serial.println(ssid);

	WiFi.begin(ssid, password);

	while (WiFi.status() != WL_CONNECTED) {
		delay(500);
		Serial.print(".");
	}
	Serial.println("");
	Serial.println("WiFi connected");

	// Start the server
	server.begin();
	Serial.println("Server started");

	// Print the IP address
	Serial.println(WiFi.localIP());
}

// ---- setup() ---------------------------------------------------------------------------------------------------------------------
void setup() {
	int iEEwritePosition;

	Serial.begin(115200);
	delay(10);

	pinMode(relaisOut, OUTPUT);
	digitalWrite(relaisOut, false);

	// EEPROM initialisieren (EEPROM-like area of Flash memory just behind SPIFF)
	EEPROM.begin(100);										// reserve 100 byte of EEPROM space
	if (EEPROM.get(0, emptyCheck) != 0x55AA)				// put() und get() beachten die Groesse der Daten (Anzahl Bytes)
	{
		iEEwritePosition = EEPROMFIRSTENTRY;				// Schreibposition auf ersten Eintrag setzen
		EEPROM.put(iEEwritePosition, maxTempAllowed);		// default Einschalttemperatur ins EEPROM schreiben
		iEEwritePosition += sizeof(maxTempAllowed);
		EEPROM.put(iEEwritePosition, TempHysterese);		// default Hysterese ins EEPROM schreiben
		iEEwritePosition += sizeof(TempHysterese);
		EEPROM.put(iEEwritePosition, iDisplayTime);			// Display OnTime ins EEPROM schreiben
		iEEwritePosition += sizeof(iDisplayTime);
		
		EEPROM.put(0, 0x55AA);								// magic number ("Werte im EEPROM sind gueltig") schreiben
		EEPROM.commit();
		delay(100);											// dem EEPROM Schreibvorgang Zeit geben
	}

	// Werte aus dem EEPROM in die Variablen schreiben
	iEEwritePosition = EEPROMFIRSTENTRY;					// Leseposition auf ersten Eintrag setzen
	EEPROM.get(iEEwritePosition, maxTempAllowed);			// Einschalttemperatur aus EEPROM lesen
	iEEwritePosition += sizeof(maxTempAllowed);
	EEPROM.get(iEEwritePosition, TempHysterese);			// Hysterese aus EEPROM lesen
	iEEwritePosition += sizeof(TempHysterese);
	EEPROM.get(iEEwritePosition, iDisplayTime);				// Display OnTime aus EEPROM lesen
	iEEwritePosition += sizeof(iDisplayTime);

	
	sensors.begin();
	sensors.setResolution(12);								// initialize DS1820 sensor with 12 bit resolution
	sensors.getAddress(sensorAddress, 0);					// Adresse des angeschlossenen Temperatursensors auslesen

	WiFiStart();											// WiFi Verbindung erstmals aufbauen

	//debugOUT("1");

	while (WiFi.status() != WL_CONNECTED) {
		delay(500);
		Serial.print(".");
	}
	Serial.println("");
	Serial.println("WiFi connected");

	//debugOUT("2");

	// Start the server
	server.begin();
	Serial.println("Server started");

	// Print the IP address
	Serial.print("Use this URL to connect: ");
	Serial.print("http://");
	Serial.print(WiFi.localIP());
	Serial.println("/");

#ifndef ESP-01
	wdt_disable();
#endif

	// Initialisierung I2C fuer OLED Display	
#ifndef DEBUG
	Wire.pins(PIN_I2C_SDA, PIN_I2C_SCL);
	Wire.begin();
#endif

	// die nachfolgenden Ausgaben auf Seriell kommen nicht mehr an, da der TX-Anschluss fr I2C verwendet wird.
	// ab jetzt kommt hier nur noch Schrott im Monitor an
	Serial.println("Initialise OLED screen...");
	StartUp_OLED();
	clear_display();
	sendStrXY("TempSchalter", 0, ZeilenOffset);				// 16 chars / line
	sendStrXY("ESP-01 + OLED", 2, ZeilenOffset);
	sendStrXY("  + DS18b20", 4, ZeilenOffset);
	//sendStrXY("IP ", 6, ZeilenOffset);
	//sendStrXY(IPadress, 6, ZeilenOffset + 4);
	delay(2000);
	clear_display();
	Serial.println("Setup Ende");

#ifndef ESP-01
	wdt_enable(WDTO_1S);
#endif

	displayTimer = millis();							// Display nach Zeit ausschalten, das OLED geht sonst kaputt
	bDisplayIsOn = true;
}

// --- loop() ----------------------------------------------------------------------------------------------------------------------------------
void loop() {
	static unsigned long looptime = 0, looptime2 = 0;
	const char ZeilenOffset = 2;							// linker Rand fuer Textausgaben auf OLED
	static int counter = 0;

	if (iDisplayTime > 0)									// mit iDisplayTime == 0 Display nie abschalten
	{
		if ((bDisplayIsOn == true) && (millis() > (displayTimer + (iDisplayTime * 1000))))				// nach 10 s Display ausschalten
		{
			displayOff();
			bDisplayIsOn = false;
		}
	}

	//debugOUT("3");										// das kommt viel zu oft und zu schnell, daher auskommentiert
	
	digitalWrite(relaisOut, value);							// Luefter abhaengig von Temperatur oder von WLAN-Eingabe schalten

	// ----------------------------------------- WEB-Server stuff -------------------------------
  
	if (WiFi.status() != WL_CONNECTED)						// check if WLAN is connected
	{
		//debugOUT("4");
		WiFiStart();										// falls nicht, Wifi starten
	}
	
	WiFiClient client = server.available();					// Check if a client has connected

	if (!client) {											// im Idle roedelt der Code in diesem Abschnitt und wartet auf einen sich anmeldenden Client - vvvv
		if (millis() > looptime + 1000) 					// jede Sekunde aktualisieren
		{
			//debugOUT("5");

			triggerTemperature();							// TemperaturMessung starten

		// Ausgabe auf OLED
#ifndef DEBUG												// wenn ich Debugausgaben auf der seriellen SS brauche, DEBUG definieren
			dtostrf(currentTemp, 2, 1, OLEDBuffer);			// aktuelle Temperatur in String wandeln
			sendStrXY("TempSchalter", 0, ZeilenOffset);		// 16 Spalten / 7 Zeilen
			sendStrXY("Temp:", 2, ZeilenOffset);
			sendStrXY(OLEDBuffer, 2, ZeilenOffset + 6);
			//sendStrXY("C", 2, 11);
			sendStrXY("Luefter:", 4, ZeilenOffset);
			strcpy(tempBuf, value == false ? "Aus" : "An ");
			sendStrXY(tempBuf, 4, ZeilenOffset + 9);
			strcpy(tempBuf, wlanOverride == false ? "automatisch" : "manuell    ");
			sendStrXY(tempBuf, 6, ZeilenOffset);

			strcpy(tempBuf,"On ");
			sendStrXY(tempBuf, 7, 0);
			dtostrf(maxTempAllowed, 2, 1, tempBuf);
			sendStr((unsigned char *)tempBuf);
			strcpy(tempBuf," Hy ");
			sendStr((unsigned char *)tempBuf);
			dtostrf(TempHysterese, 1, 1, tempBuf);
			sendStr((unsigned char *)tempBuf);
#endif
			looptime = millis();
		}

		getTemperature();									// hole Temperatur vom Sensor (nach Ablauf der Wandlungszeit)
		calcTempAndSetRelaisVar();							// setzt Variable "value" (fuer Relais) anhand der aktuellen Temperatur und den Parametern

		return;
	}														// im Idle roedelt der Code in diesem Abschnitt und wartet auf einen sich anmeldenden Client - ^^^^

	unsigned long ultimeout = millis() + 250;
	while (!client.available() && (millis() < ultimeout))
	{
		//debugOUT("6");
		delay(1);
	}

	if (millis() > ultimeout)								// wenn der Client nach 250 ms nicht da ist, Abbruch/Neustart
	{
		Serial.println("client connection time-out!");

		//debugOUT("7");
		//for (char i = 0; i < 10; i++)						// Debug
		//{
		//	digitalWrite(ledPin, !digitalRead(ledPin));
		//	delay(50);
		//}
		return;
	}

	// Wait until the client sends some data
	Serial.println("new client");

	//debugOUT("8");

	//while (!client.available()) 
	//{
	//	debugOUT("8a");

	//	if (counter++ > 100)								// Serie von "8a" im Debug Output unterteilen
	//	{
	//		Serial.print(" ---");
	//		counter = 0;
	//		return;											// dieses return nach 100 Runden unterbricht die ansonsten 
	//														//  sporadisch auftretende Endlossschleife bei Bedienung per Handy
	//	}
	//	delay(10);											// war "1", wohl nur zum Aufruf von yield()
	//}

	// Request einlesen ---------------------------------------------------------------------------------------------------
	String request = client.readStringUntil('\r');
	Serial.println(request);
	client.flush();

	// stop client, if request is empty
	if (request == "")
	{
		Serial.println("empty request! - stopping client");
		client.stop();
		//debugOUT("9");
		return;
	}

	// Request analysieren, einige Fehlerbedingungen abfangen...
	// get path; end of path is either space or ?
	// Syntax is e.g. GET /?pin=MOTOR1STOP HTTP/1.1
	String sPath = "", sParam = "", sCmd = "";
	String sGetstart = "GET ";
	int iStart, iEndSpace, iEndQuest;
	iStart = request.indexOf(sGetstart);
	if (iStart >= 0)
	{
		//debugOUT(10);

		iStart += +sGetstart.length();
		iEndSpace = request.indexOf(" ", iStart);
		iEndQuest = request.indexOf("?", iStart);

		// are there parameters?
		if (iEndSpace > 0)
		{
			if (iEndQuest > 0)
			{
				// there are parameters
				sPath = request.substring(iStart, iEndQuest);
				sParam = request.substring(iEndQuest, iEndSpace);
			}
			else
			{
				// NO parameters
				sPath = request.substring(iStart, iEndSpace);
			}
		}
	}

	//// optional - Bedienung einer seriellen Schnittstelle
	////////////////////////////////////////////////////////////////////////////////////
	//// output parameters to serial, you may connect e.g. an Arduino and react on it //
	////////////////////////////////////////////////////////////////////////////////////
	//if (sParam.length() > 0)
	//{
	//	int iEqu = sParam.indexOf("=");
	//	if (iEqu >= 0)
	//	{
	//		sCmd = sParam.substring(iEqu + 1, sParam.length());
	//		Serial.println(sCmd);
	//	}
	//}

	/////////////////////////////////////////
	// 404 angegebener Pfad nicht gefunden //
	/////////////////////////////////////////
	if (sPath != "/")
	{
		String sResponse = "<html><head><title>404 Not Found</title></head>"
					 "<body><h1>Not Found</h1>"
					 "<p>The requested URL was not found on this server.</p>"
					 "</body></html>";
		String sHeader = "HTTP/1.1 404 Not found\r\n";
		sHeader += "Content-Length: ";
		sHeader += sResponse.length();
		sHeader += "\r\n";
		sHeader += "Content-Type: text/html\r\n";
		sHeader += "Connection: close\r\n";
		sHeader += "\r\n";

		client.print(sHeader);
		client.print(sResponse);
	}

	// pruefen ob Schluesselworte im Request vorkommen
	if (request.indexOf("LEDEIN") != -1) {
		value = true;										// Relais manuell ein
		wlanOverride = true;								// Relais wurde ueber WLAN geschaltet
	}
	else if (request.indexOf("LEDAUS") != -1) {
		value = false;										// Relais manuell aus
		wlanOverride = true;								// Relais wurde ueber WLAN geschaltet
	}
	else if (request.indexOf("LEDAUTO") != -1) {
		wlanOverride = false;								// Relais wird wieder temperaturabhaengig geschaltet
	}

	// Settings Bildschirm anzeigen/ausblenden
	if (request.indexOf("SETTINGSON") != -1) {
		bEinstellungen = true;								// Einstellungen anzeigen
	}
	if (request.indexOf("SETTINGSOFF") != -1) {
		bEinstellungen = false;								// Einstellungen ausblenden
	}

	// Einschalttemperatur setzen
	if (request.indexOf("25GRAD") != -1) {
		maxTempAllowed = 25.0;								// Einschalttemperatur setzen
		cEEPROMchanged = TEMP;
	}
	if (request.indexOf("30GRAD") != -1) {
		maxTempAllowed = 30.0;								// Einschalttemperatur setzen
		cEEPROMchanged = TEMP;
	}
	if (request.indexOf("PLUS1") != -1) {
		maxTempAllowed += 1;								// Einschalttemperatur um 1 Grad verringern
		cEEPROMchanged = TEMP;
	}
	if (request.indexOf("MINUS1") != -1) {
		maxTempAllowed -= 1;								// Einschalttemperatur um 1 Grad erhoehen
		cEEPROMchanged = TEMP;
	}

	// Hysterese setzen
	if (request.indexOf("KOMMA5GRAD") != -1) {
		TempHysterese = 0.5;								// Hysterese setzen
		cEEPROMchanged = HYST;
	}
	if (request.indexOf("1GRAD") != -1) {
		TempHysterese = 1.0;								// Hysterese setzen
		cEEPROMchanged = HYST;
	}
	if (request.indexOf("1KOMMA5") != -1) {
		TempHysterese = 1.5;								// Hysterese setzen
		cEEPROMchanged = HYST;
	}
	if (request.indexOf("2GRAD") != -1) {
		TempHysterese = 2.0;								// Hysterese setzen
		cEEPROMchanged = HYST;
	}

	if (request.indexOf("DISP3SEK") != -1) {
		iDisplayTime = 3;								// Displaytimer setzen
		cEEPROMchanged = DISP;
	}
	if (request.indexOf("DISP10SEK") != -1) {
		iDisplayTime = 10;								// Displaytimer setzen
		cEEPROMchanged = DISP;
	}
	if (request.indexOf("DISP30SEK") != -1) {
		iDisplayTime = 30;								// Displaytimer setzen
		cEEPROMchanged = DISP;
	}
	if (request.indexOf("DISP60SEK") != -1) {
		iDisplayTime = 60;								// Displaytimer setzen
		cEEPROMchanged = DISP;
	}
	if (request.indexOf("DISPAN") != -1) {				// Display immer an
		iDisplayTime = 0;								// Displaytimer setzen
		cEEPROMchanged = DISP;
	}

	// ggf. erfolgte Aenderungen ins EPROM uebernehmen
	if (cEEPROMchanged == TEMP)								// Einschalttemperatur geaendert
	{
		digitalWrite(ledPin, !digitalRead(ledPin));			// kurzer Blip als Nachricht an User
		EEPROM.put(EEPROMFIRSTENTRY, maxTempAllowed);
		EEPROM.commit();									// nach Aenderungen den Inhalt des temporaeren RAM ins Flash schreiben
		delay(200);											// dem EEPROM Schreibvorgang Zeit geben
		cEEPROMchanged = NOCHANGE;
		digitalWrite(ledPin, !digitalRead(ledPin));			// kurzer Blip als Nachricht an User
	}
	if (cEEPROMchanged == HYST)								// Hysterese geaendert
	{
		digitalWrite(ledPin, !digitalRead(ledPin));			// kurzer Blip als Nachricht an User
		EEPROM.put(EEPROMFIRSTENTRY + sizeof(maxTempAllowed), TempHysterese);
		EEPROM.commit();									// nach Aenderungen den Inhalt des temporaeren RAM ins Flash schreiben
		delay(200);											// dem EEPROM Schreibvorgang Zeit geben
		cEEPROMchanged = 0;
		digitalWrite(ledPin, !digitalRead(ledPin));			// kurzer Blip als Nachricht an User
	}
	if (cEEPROMchanged == DISP)								// Einschalttemperatur geaendert
	{
		digitalWrite(ledPin, !digitalRead(ledPin));			// kurzer Blip als Nachricht an User
		EEPROM.put(EEPROMFIRSTENTRY + sizeof(maxTempAllowed) + sizeof(TempHysterese), iDisplayTime);
		EEPROM.commit();									// nach Aenderungen den Inhalt des temporaeren RAM ins Flash schreiben
		delay(200);											// dem EEPROM Schreibvorgang Zeit geben
		cEEPROMchanged = NOCHANGE;
		digitalWrite(ledPin, !digitalRead(ledPin));			// kurzer Blip als Nachricht an User
	}
	   
	// Antwort an den Client (Webbrowser) zurueckgeben
	client.println("HTTP/1.1 200 OK");
	client.println("Content-Type: text/html");
	client.println(""); //  do not forget this one
	client.println("<!DOCTYPE HTML>");
	client.println("<html>");																	// html Start

	// hiermit koennte der Browserinhalt automatisch aktualisiert werden
	//client.println("<meta http-equiv=\"refresh\" content=\"10\">"); //aktualisiere page alle 10 sec

	client.println("<br><br>");
	// eigene Ausgaben
	client.print("<html><head><title>L&uuml;fter-Steuerung</title></head><body>");
	client.print("<font color=\"#000000\"><body bgcolor=\"#d0d0f0\">");
	client.print("<meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0, user-scalable=yes\">");
	client.print("<h1>L&uuml;fter-Steuerung</h1>");

	client.print("<FONT SIZE=+1>");
	client.print("Temperatur: ");
	client.print(getContent());
	client.print(" &deg;C");
	client.print("<BR><BR>");
	client.print("L&uumlfter ");

	if (wlanOverride == false) { client.print("auto - "); 
	}
	else { client.print("manuell - "); 
	}

	if (value == true) { client.print("ein"); 
	}
	else {client.print("aus");
	}

	client.print("<BR>");

	//client.print("<FONT SIZE=-1>");
	client.print("<p>L&uuml;fter <a href=\"?pin=LEDEIN\"><button>ein</button></a>&nbsp;"
				 "<a href=\"?pin=LEDAUS\"><button>aus</button></a>&nbsp;"
				 "<a href=\"?pin=LEDAUTO\"><button>auto</button></a></p>");

	// Bildschirminhalt fuer Einstellungen ein/ausblenden
	if (bEinstellungen == false)								// Button Beschriftung definieren
	{
		client.print("<p><a href=\"?pin=SETTINGSON\"><button>Einstellungen</button></a>&nbsp;");
	}
	else
	{
		client.print("<p><a href=\"?pin=SETTINGSOFF\"><button>Einstellungen verlassen</button></a>&nbsp;");
	}
	client.print("<BR>");

	if (bEinstellungen == true)									// Einstellungen anzeigen
	{
		client.print("<p>Einschalttemperatur aktuell:&nbsp;");
		client.print(maxTempAllowed);
		client.print("&nbsp;&nbsp;Hysterese:&nbsp;");
		client.print(TempHysterese);
		client.print("<BR><\p>");

		client.print("<p>L&uuml;fter einschalten bei");
		client.print("<BR>");
		client.print("<a href=\"?pin=25GRAD\"><button>25 &deg;C</button></a>&nbsp;"
			"<a href=\"?pin=30GRAD\"><button>30 &deg;C</button></a>&nbsp;"
			"<a href=\"?pin=PLUS1\"><button>+1</button></a>&nbsp;"
			"<a href=\"?pin=MINUS1\"><button>-1</button></a></p>");

		//client.print("<BR>");
		client.print("<p>Hysterese");
		client.print("<BR>");
		client.print("<a href=\"?pin=KOMMA5GRAD\"><button>0.5 K</button></a>&nbsp;"
			"<a href=\"?pin=1GRAD\"><button>1 K</button></a>&nbsp;"
			"<a href=\"?pin=1KOMMA5\"><button>1.5 K</button></a>&nbsp;"
			"<a href=\"?pin=2GRAD\"><button>2 K</button></a></p>");

		client.print("<p>Displaytimer: &nbsp;");
		client.print(iDisplayTime);
		client.print("<BR>");
		client.print("<a href=\"?pin=DISP3SEK\"><button>3 s</button></a>&nbsp;"
			"<a href=\"?pin=DISP10SEK\"><button>10 s</button></a>&nbsp;"
			"<a href=\"?pin=DISP30SEK\"><button>30 s</button></a>&nbsp;"
			"<a href=\"?pin=DISP60SEK\"><button>60 s</button></a>&nbsp;"
			"<a href=\"?pin=DISPAN\"><button>immer an</button></a></p>");
	}

	client.println("</html>");																	// html Ende

	delay(1);
	Serial.println("Client disonnected");
	Serial.println("");

	displayOn();											// Display nach Empfang vom WEB einschalten
	bDisplayIsOn = true;
	displayTimer = millis();								// Display nach Zeit ausschalten, das OLED geht sonst kaputt
}
// --- loop() ----------------------------------------------------------------------------------------------------------------------------------


void debugOUT(String code)
{
	Serial.print(code);
}